==> trip-0.1.0.tar.gz: trip-0.1.0/tripstack/tripcore/TRIPItadPathSegment.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "TRIPItadPathSegment.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPItadPathSegment::TRIPItadPathSegment(TRIPPathSegType type)
{
    assert((type > 0) && (type < TRIP_SEG_MAX));
    _type = type;
    _length = 0;
}

TRIPItadPathSegment::TRIPItadPathSegment(TRIPPathSegType type, u_int16_t itad)
{
    assert((type > 0) && (type < TRIP_SEG_MAX));
    _type = type;
    _length = 0;
    addItad(itad);
}

TRIPItadPathSegment::TRIPItadPathSegment(const char* buf) throw(TRIPBadDataException&)
{
    u_int8_t type;
    u_int8_t  len;
    memcpy(&type, buf, 1);
    memcpy(&len, buf+1, 1);
    _type = static_cast<TRIPPathSegType>(type); 
    _length = len; 
    if((_type <= 0) || (_type >= TRIP_SEG_MAX) || ((len / 2) != 0) )
    {
        cpLog(LOG_ALERT, "Bad path segment data");
        throw TRIPBadDataException("Bad path segment data", __FILE__, __LINE__, 0);
    }

    assert((len / 2) == 0);
    _length = 0;
    _itadList.clear();

    char* tBuf = new char[len*2];
    memcpy(tBuf, buf+2, len*2);
    int idx = 0;
    for(int i = 0; i < len; i++)
    {
        u_int16_t tt;
        memcpy(&tt, tBuf+idx, 2);
        tt = ntohs(tt);
        _itadList.push_back(tt);
        idx += 2;
    }
    _length = _itadList.size();
    delete [] tBuf;
}

TRIPItadPathSegment::TRIPItadPathSegment(const TRIPItadPathSegment& other) 
{
    copyObj(other);    
}

TRIPItadPathSegment& 
TRIPItadPathSegment::operator=(const TRIPItadPathSegment& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPItadPathSegment::copyObj(const TRIPItadPathSegment& other)
{
    _type = other._type;
    _length = other._length;
    _itadList.clear();
    ITAD_LIST::const_iterator itr;
    for(itr = other._itadList.begin(); 
              itr != other._itadList.end(); itr++)
    {
        _itadList.push_back(*itr);
    }
    _length = _itadList.size();
}

void
TRIPItadPathSegment::addItad(u_int16_t itad)
{
    _itadList.push_back(itad);
    _length++;
}

void 
TRIPItadPathSegment::writeData(int id) const throw(VNetworkException&)
{
    struct shipIt {
        u_int8_t type;
        u_int8_t len;
    } sIt;

    sIt.type = _type;
    sIt.len = _length;

    TRIPUtil::writeData(id, &sIt, sizeof(struct shipIt));

    char* tBuf = new char[_length*2];
    int idx = 0;
    ITAD_LIST::const_iterator itr;
    for(itr = _itadList.begin(); itr != _itadList.end(); itr++)
    {
        u_int16_t tt = (*itr);
        tt = htons(tt);
        memcpy(tBuf+idx, &tt, 2);
        idx += 2;
    }

    TRIPUtil::writeData(id, tBuf, _length*2);
    delete [] tBuf;
}

==> trip-1.0.0.tar.gz: trip-1.0.0/tripstack/tripcore/TRIPItadPathSegment.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <netinet/in.h>

#include "TRIPItadPathSegment.hxx"
#include "TRIPUtil.hxx"
#include "TRIPErrorHandler.hxx"
#include "cpLog.h"

TRIPItadPathSegment::TRIPItadPathSegment(TRIPPathSegType type)
{
    assert((type >= 1) && (type < TRIP_SEG_MAX));
    _type = type;
    _numOfItads = 0;
}

TRIPItadPathSegment::TRIPItadPathSegment(TRIPPathSegType type, u_int16_t itad)
{
    assert((type >= 1) && (type < TRIP_SEG_MAX));
    _type = type;
    _numOfItads = 0;
    addItad(itad);
}

TRIPItadPathSegment::TRIPItadPathSegment(const char* buf) throw(TRIPBadDataException&)
{
    u_int8_t type;
    u_int8_t  numOfItads;
    memcpy(&type, buf, 1);
    memcpy(&numOfItads, buf+1, 1);
    _type = static_cast<TRIPPathSegType>(type); 

    if((_type <= 0) || (_type >= TRIP_SEG_MAX))
    {
        throw TRIPBadDataException("Bad path segment data", 
                                        __FILE__, __LINE__, TRIP_ERR_UPDATE);
    }

    _itadList.clear();

    const char* tBuf = buf+2;
    for(int i = 0, idx=0; i < numOfItads; i++, idx = idx+2)
    {
        u_int16_t tt;
        memcpy(&tt, tBuf+idx, 2);
        tt = ntohs(tt);
        _itadList.push_back(tt);
    }
    _numOfItads = _itadList.size();
}

TRIPItadPathSegment::TRIPItadPathSegment(const TRIPItadPathSegment& other) 
{
    copyObj(other);    
}

TRIPItadPathSegment& 
TRIPItadPathSegment::operator=(const TRIPItadPathSegment& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPItadPathSegment::copyObj(const TRIPItadPathSegment& other)
{
    _type = other._type;
    _itadList.clear();
    ITAD_LIST::const_iterator itr;
    for(itr = other._itadList.begin(); 
              itr != other._itadList.end(); itr++)
    {
        _itadList.push_back(*itr);
    }
    _numOfItads = _itadList.size();
}

void
TRIPItadPathSegment::addItad(u_int16_t itad)
{
    _itadList.push_back(itad);
    _numOfItads++;
}

void 
TRIPItadPathSegment::writeData(int id) const throw(VNetworkException&)
{
    struct shipIt {
        u_int8_t type;
        u_int8_t len;
    } sIt;

    sIt.type = _type;
    sIt.len = _numOfItads;

    TRIPUtil::writeData(id, &sIt, sizeof(struct shipIt));

    char* tBuf = new char[_numOfItads*2];
    int idx = 0;
    for(ITAD_LIST::const_iterator itr = _itadList.begin() ; 
              itr != _itadList.end(); itr++, idx = idx +2)
    {
        u_int16_t tt = (*itr);
        tt = htons(tt);
        memcpy(tBuf+idx, &tt, 2);
    }

    TRIPUtil::writeData(id, tBuf, _numOfItads*2);
    delete [] tBuf;
}


bool 
TRIPItadPathSegment::operator==(const TRIPItadPathSegment& other) const 
{
    return((_type == other._type) &&
           (_numOfItads == other._numOfItads) &&
           (_itadList == other._itadList));
}

bool 
TRIPItadPathSegment::containsItad(u_int16_t itad) const
{
    ITAD_LIST::const_iterator itr;
    for(itr = _itadList.begin(); itr != _itadList.end(); itr++)
    {
        if((*itr) == itad) return true; 
    }
    return false;
}

bool 
TRIPItadPathSegment::isEmpty() const
{
    return (_itadList.size() == 0);
}

void 
TRIPItadPathSegment::setType(TRIPPathSegType type)
{
    assert((type >= 0) && (type < TRIP_SEG_MAX));
    _type = type;
}
